<?php
use JetBrains\PhpStorm\NoReturn;

require 'app-config.php';

session_start([
    'name'            => 'FRCSESSION',
    'use_strict_mode' => true,
    'cookie_httponly' => true,
    'cookie_samesite' => 'Strict']);

/**
 * Add a message to be displayed at the top of the page
 *
 * @param string $level The level (type) of the message
 * @param string $message The message itself
 */
function add_message(string $level, string $message): void {
    if (!key_exists(Key::USER_MSG, $_SESSION)) $_SESSION[Key::USER_MSG] = array();
    $_SESSION[Key::USER_MSG][] = ['level' => $level, 'message' => $message];
}

/**
 * Add an error message to be displayed at the top of the page
 *
 * @param string $message The message to be displayed
 */
function add_error(string $message): void {
    add_message('ERROR', $message);
}

/**
 * Add an error message to be displayed at the top of the page
 *
 * @param string $message The message to be displayed
 */
function add_info(string $message): void {
    add_message('INFO', $message);
}

/** @var bool $is_htmx True if this request was initiated by htmx, false if not */
$is_htmx = key_exists('HTTP_HX_REQUEST', $_SERVER) && !key_exists('HTTP_HX_HISTORY_RESTORE_REQUEST', $_SERVER);

function nav_link(string $link, bool $isFirst = false) {
    $sep = $isFirst ? '' : ' | ';
    echo "<span>$sep$link</span>";
}

/**
 * Render the title bar for the page
 */
function title_bar(): void {
    $version = match (true) {
        str_ends_with(FRC_VERSION, '.0.0') => substr(FRC_VERSION, 0, strlen(FRC_VERSION) - 4),
        str_ends_with(FRC_VERSION, '.0')   => substr(FRC_VERSION, 0, strlen(FRC_VERSION) - 2),
        default                            => FRC_VERSION
    }; ?>
    <header hx-target=#main hx-push-url=true>
        <div><a href=/ class=title>Feed Reader Central</a><span class=version>v<?=$version?></span></div>
        <nav><?php
            if (key_exists(Key::USER_ID, $_SESSION)) {
                $db = Data::getConnection();
                try {
                    $bookQuery = $db->prepare(<<<'SQL'
                        SELECT EXISTS(
                            SELECT 1
                              FROM item INNER JOIN feed ON item.feed_id = feed.id
                             WHERE feed.user_id = :id AND item.is_bookmarked = 1)
                        SQL);
                    $bookQuery->bindValue(':id', $_SESSION[Key::USER_ID]);
                    $bookResult   = $bookQuery->execute();
                    $hasBookmarks = $bookResult && $bookResult->fetchArray(SQLITE3_NUM)[0];
                    nav_link(hx_get('/feeds', 'Feeds'), true);
                    if ($hasBookmarks) nav_link(hx_get('/?bookmarked', 'Bookmarked'));
                    nav_link(hx_get('/search', 'Search'));
                    nav_link(hx_get('/docs/', 'Docs'));
                    nav_link('<a href=/user/log-off>Log Off</a>');
                    if ($_SESSION[Key::USER_EMAIL] != Security::SINGLE_USER_EMAIL) {
                        nav_link($_SESSION[Key::USER_EMAIL]);
                    }
                } finally {
                    $db->close();
                }
            } else {
                nav_link(hx_get('/user/log-on', 'Log On'), true);
                nav_link(hx_get('/docs/', 'Docs'));
            } ?>
        </nav>
    </header>
    <main id=main hx-target=this hx-push-url=true hx-swap="innerHTML show:window:top"><?php
}

/**
 * Render the page title
 * @param string $title The title of the page being displayed
 */
function page_head(string $title): void {
    global $is_htmx;
    ?>
<!DOCTYPE html>
<html lang=en>
<head>
    <title><?=$title?> | Feed Reader Central</title><?php
    if (!$is_htmx) { ?>
        <meta name=viewport content="width=device-width, initial-scale=1">
        <meta name=htmx-config content='{"historyCacheSize":0}'>
        <link href=/assets/style.css rel=stylesheet><?php
    } ?>
</head>
<body><?php
    if (!$is_htmx) title_bar();
    if (sizeof($messages = $_SESSION[Key::USER_MSG] ?? []) > 0) { ?>
        <div class=user_messages><?php
            array_walk($messages, function ($msg) { ?>
                <div class=user_message>
                    <?=$msg['level'] == 'INFO' ? '' : "<strong>{$msg['level']}</strong><br>"?>
                    <?=$msg['message']?>
                </div><?php
            }); ?>
        </div><?php
        $_SESSION[Key::USER_MSG] = [];
    }
}

/**
 * Render the end of the page
 */
function page_foot(): void {
    global $is_htmx; ?>
    </main><?php
    if (!$is_htmx) echo '<script src=/assets/htmx.min.js></script>'; ?>
</body>
</html><?php
    session_commit();
}

/**
 * Redirect the user to the given URL
 *
 * @param string $value A local URL to which the user should be redirected
 */
#[NoReturn]
function frc_redirect(string $value): void {
    if (str_starts_with($value, 'http')) {
        http_response_code(400);
        die();
    }
    session_commit();
    header("Location: $value", true, 303);
    die();
}

/**
 * Convert a date/time string to a date/time in the configured format
 *
 * @param string $value The date/time string
 * @return string The standard format of a date/time, or '(invalid date)' if the date could not be parsed
 */
function date_time(string $value): string {
    try {
        return (new DateTimeImmutable($value))->format(DATE_TIME_FORMAT);
    } catch (Exception) {
        return '(invalid date)';
    }
}

/**
 * Create an anchor tag with both `href` and `hx-get` attributes
 *
 * @param string $url The URL to which navigation should occur
 * @param string $text The text for the link
 * @param string $extraAttrs Extra attributes for the anchor tag (must be attribute-encoded)
 * @return string The anchor tag with both `href` and `hx-get` attributes
 */
function hx_get(string $url, string $text, string $extraAttrs = ''): string {
    $attrs = $extraAttrs != '' ? " $extraAttrs" : '';
    return "<a href=\"$url\" hx-get=\"$url\"$attrs>$text</a>";
}

/**
 * Return a 404 Not Found
 */
#[NoReturn]
function not_found(): void {
    http_response_code(404);
    die('Not Found');
}
